//##############################################################
// Das Flag bDirty dient dazu festzulaten, dass die Eingabe
// noch nicht verifiziert wurde. Aufgrund der Reihenfolge-
// problematik wird in der Klasse Datum sptestens vor einer 
// Herausgabe der Daten geprft, ob gecheckt werden muss.
// somit ist es egal, in welcher Reihenfolge die Methoden
// setTag(int pTag), setMonat(int pMonat) und setjahr(int pJahr)
// aufgerufen werden
//##############################################################
#include "stdafx.h"
#include <time.h>
#include <strstream>
#include "hg_datum.h"

using namespace std;

//==============================================================
// Standardconstructor, belegt das Datumsobjekt mit dem heutigen
// Datum (gem PC-Uhr)
//==============================================================
HG::Datum::Datum (void)
{
  setHeute();
}

//==============================================================
// Constructor, mit Vorgaben
//==============================================================
HG::Datum::Datum (int pTag, int pMonat, int pJahr)
{
  set(pTag, pMonat, pJahr);
}

//==============================================================
// Copy-Constructor (Referenz)
//==============================================================
HG::Datum::Datum (Datum &pDatum)
{
  nTag   = pDatum.nTag;
  nMonat = pDatum.nMonat;
  nJahr  = pDatum.nJahr;
  bDirty = pDatum.bDirty;
}

//==============================================================
// Copy-Constructor (Pointer)
//==============================================================
HG::Datum::Datum (Datum *ptrDatum)
{
  if (ptrDatum)
  {
    nTag   = ptrDatum->nTag;
    nMonat = ptrDatum->nMonat;
    nJahr  = ptrDatum->nJahr;
    bDirty = ptrDatum->bDirty;
  }
  else setHeute();
}

//==============================================================
// Inhalt des Datums auf den aktuellen Tag gem PC-Uhr setzen
//==============================================================
void HG::Datum::setHeute (void)
{
  //------------------------------------------------------------
  // Dirtyflag zurcksetzen
  //------------------------------------------------------------
  bDirty = false;

  //------------------------------------------------------------
  // Timerticks aus PC-Uhr holen
  //------------------------------------------------------------
  time_t nTimerTicks;
  time(&nTimerTicks);                

  //------------------------------------------------------------
  // Konvertierung der Timerticks in das gem PC eingestellte
  // Zeitschema
  //------------------------------------------------------------
  struct tm *rcZeit = localtime(&nTimerTicks); 

  //------------------------------------------------------------
  // Aus der Struktur die relevanten Informationen herausnehmen
  // tm_mday = Tag im Monat, (1  31)
  // tm_mon  = Monat im Jahr (0  11; January = 0) (!!!!!) 
  // tm_year = Jahr          (Aktuelles Jahr minus 1900)
  //------------------------------------------------------------
  nTag   = rcZeit->tm_mday;             
  nMonat = rcZeit->tm_mon  + 1;             
  nJahr  = rcZeit->tm_year + 1900;
}

//==============================================================
// Getter des Members nTag
//==============================================================
int HG::Datum::getTag (void)
{
  if (bDirty) correct();
  return nTag;
}

//==============================================================
// Setter des Members nTag
//==============================================================
void HG::Datum::setTag (int pTag)
{
  bDirty = true;
  nTag = pTag;
}

//==============================================================
// Getter des Members nMonat
//==============================================================
int HG::Datum::getMonat (void)
{
  if (bDirty) correct();
  return nMonat;
}

//==============================================================
// Setter des Members nMonat
//==============================================================
void HG::Datum::setMonat (int pMonat)
{
  bDirty = true;
  nMonat = pMonat;
}

//==============================================================
// Getter des Members nJahr
//==============================================================
int HG::Datum::getJahr (void)
{
  if (bDirty) correct();
  return nJahr;
}

//==============================================================
// Setter des Members nJahr
//==============================================================
void HG::Datum::setJahr (int pJahr)
{
  bDirty = true;
  nJahr = pJahr;
}

//==============================================================
// ganzes Datum auf einmal setzen
//==============================================================
void HG::Datum::set (int pTag, int pMonat, int pJahr)
{
  nTag   = pTag;
  nMonat = pMonat;
  nJahr  = pJahr;
  correct();
}

//==============================================================
// ganzes Datum als long-Wert zurckgeben
//==============================================================
long HG::Datum::getLong (void)
{
  if (bDirty) correct();
  return nJahr + nMonat*10000 + nTag * 1000000;
}

//==============================================================
// ganzes Datum als sortierbaren long-Wert zurckgeben
//==============================================================
long HG::Datum::getLongSortierbar (void)
{
  if (bDirty) correct();
  return nTag + nMonat*100 + nJahr * 10000;
}

//==============================================================
// Datumsregeln prfen
//==============================================================
void HG::Datum::correct (void)
{
  //------------------------------------------------------------
  // Dirtyflag zurcksetzen
  //------------------------------------------------------------
  bDirty = false;

  //------------------------------------------------------------
  // Korrektur: Jahreszahl gltig machen. Das Gregorianische
  // Kalendersystem bersprang 1582 ganze 10 Tage (auf den 
  // 04.10. folgte der 15.10.), um den rechnerischen Kalender
  // (bis dahin Julianisch) wieder mit der Frhlings-
  // Tagundnachtgleiche am 21. Mrz in Einklang zu bringen.
  // Auch dies ist natrlich nur eine Annahme, tatschlich ist
  // der Gregorianische Kalender nicht berall zur gleichen Zeit
  // eingefhrt worden. In groen Teilen Deutschlands erst im 
  // Jahr 1700, zuletzt in der Trkei (1926)
  //------------------------------------------------------------
  if (nJahr < 1583) nJahr = 1583;
  else if (nJahr > 9999) nJahr = 9999;

  //------------------------------------------------------------
  // Korrektur: Monatszahl gltig machen 
  //------------------------------------------------------------
  if (nMonat < 1) nMonat = 1;
  else if (nMonat > 12) nMonat = 12;

  //------------------------------------------------------------
  // Vorbereitung: Wieviele Tage der Monat haben kann
  //------------------------------------------------------------
  int nMaxTageImMonat = 31;
  switch (nMonat)
  {
    case  2: nMaxTageImMonat = isSchaltjahr(nJahr) ? 29 : 28; break;
    case  4:
    case  6:
    case  9:
    case 11: nMaxTageImMonat = 30; break;
  }

  //------------------------------------------------------------
  // Korrektur: Tageszahl gltig machen 
  //------------------------------------------------------------
  if (nTag < 1) nTag = 1;
  else if (nTag > nMaxTageImMonat) nTag = nMaxTageImMonat;
}

//==============================================================
// Ausgelagerte Schaltjahresregel, kann auch von ausserhalb 
// aufgerufen werden
//==============================================================
bool HG::Datum::isSchaltjahr (int pJahr)
{
  //------------------------------------------------------------
  // Vorbereitung: feststellen, ob es ein Schaltjahr ist
  //------------------------------------------------------------
  if (nJahr % 4 == 0)
  {
    if (nJahr % 100 == 0) // Prfung auf Skularjahr
    {
      if (nJahr % 400 == 0) return true;
    }
    else return true;
  }
  return false;
}

//==============================================================
// Rckgabe als Zeichekette
//==============================================================
string HG::Datum::getString (void)
{
  char cBuffer[11];
  strstream s (cBuffer, 11);

  if (nTag < 10) s << '0';
  s << nTag << '.';
  if (nMonat < 10) s << '0';
  s << nMonat << '.' << nJahr << '\0';

  return s.str();
}
