#include "stdafx.h"
#include <time.h>
#include <iostream>
#include <iomanip>
#include "hg_datum.h"

using namespace std;

char HG::Datum::cSeperator = '.';

//==============================================================
// Standardconstructor, belegt das Datumsobjekt mit dem heutigen
// Datum (gem PC-Uhr)
//==============================================================
HG::Datum::Datum (void)
{
  setHeute();
}

//==============================================================
// Constructor, mit Vorgaben
//==============================================================
HG::Datum::Datum (int pTag, int pMonat, int pJahr)
{
  set(pTag, pMonat, pJahr);
}

//==============================================================
// Copy-Constructor (Referenz)
//==============================================================
HG::Datum::Datum (Datum &pDatum)
{
  nTag   = pDatum.nTag;
  nMonat = pDatum.nMonat;
  nJahr  = pDatum.nJahr;
}

//==============================================================
// Copy-Constructor (Pointer)
//==============================================================
HG::Datum::Datum (Datum *ptrDatum)
{
  if (ptrDatum)
  {
    nTag   = ptrDatum->nTag;
    nMonat = ptrDatum->nMonat;
    nJahr  = ptrDatum->nJahr;
  }
  else setHeute();
}

//==============================================================
// Inhalt des Datums auf den aktuellen Tag gem PC-Uhr setzen
//==============================================================
void HG::Datum::setHeute (void)
{
  //------------------------------------------------------------
  // Timerticks aus PC-Uhr holen
  //------------------------------------------------------------
  time_t nTimerTicks;
  time(&nTimerTicks);                

  //------------------------------------------------------------
  // Konvertierung der Timerticks in das gem PC eingestellte
  // Zeitschema
  //------------------------------------------------------------
  struct tm *rcZeit = localtime(&nTimerTicks); 

  //------------------------------------------------------------
  // Aus der Struktur die relevanten Informationen herausnehmen
  // tm_mday = Tag im Monat, (1  31)
  // tm_mon  = Monat im Jahr (0  11; January = 0) (!!!!!) 
  // tm_year = Jahr          (Aktuelles Jahr minus 1900)
  //------------------------------------------------------------
  nTag   = rcZeit->tm_mday;             
  nMonat = rcZeit->tm_mon  + 1;             
  nJahr  = rcZeit->tm_year + 1900;
}

//==============================================================
// Getter des Members nTag
//==============================================================
int HG::Datum::getTag (void)
{
  return nTag;
}

//==============================================================
// Setter des Members nTag
//==============================================================
void HG::Datum::setTag (int pTag)
{
  nTag = pTag;
  correct();
}

//==============================================================
// Getter des Members nMonat
//==============================================================
int HG::Datum::getMonat (void)
{
  return nMonat;
}

//==============================================================
// Setter des Members nMonat
//==============================================================
void HG::Datum::setMonat (int pMonat)
{
  nMonat = pMonat;
  correct();
}

//==============================================================
// Getter des Members nJahr
//==============================================================
int HG::Datum::getJahr (void)
{
  return nJahr;
}

//==============================================================
// Setter des Members nJahr
//==============================================================
void HG::Datum::setJahr (int pJahr)
{
  nJahr = pJahr;
  correct();
}

//==============================================================
// ganzes Datum auf einmal setzen
//==============================================================
void HG::Datum::set (int pTag, int pMonat, int pJahr)
{
  nTag   = pTag;
  nMonat = pMonat;
  nJahr  = pJahr;
  correct();
}

//==============================================================
// ganzes Datum als long-Wert zurckgeben
//==============================================================
long HG::Datum::getLong (void)
{
  return nJahr + nMonat*10000 + nTag * 1000000;
}

//==============================================================
// ganzes Datum als sortierbaren long-Wert zurckgeben
//==============================================================
long HG::Datum::getLongSortierbar (void)
{
  return nTag + nMonat*100 + nJahr * 10000;
}

//==============================================================
// Datumsregeln prfen
//==============================================================
void HG::Datum::correct (void)
{
  //------------------------------------------------------------
  // Korrektur: Jahreszahl gltig machen. Das Gregorianische
  // Kalendersystem bersprang 1582 ganze 10 Tage (auf den 
  // 04.10. folgte der 15.10.), um den rechnerischen Kalender
  // (bis dahin Julianisch) wieder mit der Frhlings-
  // Tagundnachtgleiche am 21. Mrz in Einklang zu bringen.
  // Auch dies ist natrlich nur eine Annahme, tatschlich ist
  // der Gregorianische Kalender nicht berall zur gleichen Zeit
  // eingefhrt worden. In groen Teilen Deutschlands erst im 
  // Jahr 1700, zuletzt in der Trkei (1926)
  //------------------------------------------------------------
  if (nJahr < 1583) nJahr = 1583;
  else if (nJahr > 9999) nJahr = 9999;

  //------------------------------------------------------------
  // Korrektur: Monatszahl gltig machen 
  //------------------------------------------------------------
  if (nMonat < 1) nMonat = 1;
  else if (nMonat > 12) nMonat = 12;

  //------------------------------------------------------------
  // Vorbereitung: Wieviele Tage der Monat haben kann
  //------------------------------------------------------------
  int nMaxTageImMonat = 31;
  switch (nMonat)
  {
    case  2: nMaxTageImMonat = isSchaltjahr(nJahr) ? 29 : 28; break;
    case  4:
    case  6:
    case  9:
    case 11: nMaxTageImMonat = 30; break;
  }

  //------------------------------------------------------------
  // Korrektur: Tageszahl gltig machen 
  //------------------------------------------------------------
  if (nTag < 1) nTag = 1;
  else if (nTag > nMaxTageImMonat) nTag = nMaxTageImMonat;
}

//==============================================================
// Ausgelagerte Schaltjahresregel, kann auch von ausserhalb 
// aufgerufen werden
//==============================================================
bool HG::Datum::isSchaltjahr (int pJahr)
{
  //------------------------------------------------------------
  // Vorbereitung: feststellen, ob es ein Schaltjahr ist
  //------------------------------------------------------------
  if (nJahr % 4 == 0)
  {
    if (nJahr % 100 == 0) // Prfung auf Skularjahr
    {
      if (nJahr % 400 == 0) return true;
    }
    else return true;
  }
  return false;
}

//==============================================================
// Operator: Prinkrement
// Also erst das Objekt verndern und dann eine Kopie erstellen. 
// Prinkrement verndert das Objekt erst vor der Auswertung
//==============================================================
HG::Datum HG::Datum::operator++ (void)
{
  setJahr (nJahr+1);
  return *this;
}

//==============================================================
// Operator: Postincrement
// Also erst eine Kopie erstellen, dann das Objekt selbst 
// abndern, die Kopie zurckgeben. Postinkrement verndert
// das Objekt erst nach der Auswertung
//==============================================================
HG::Datum HG::Datum::operator++ (int)
{
  HG::Datum oTemp = *this; 
  setJahr(nJahr+1);
  return oTemp;
}

//==============================================================
// Operator: Prdekrement
// Also erst das Objekt verndern und dann eine Kopie erstellen. 
// Prinkrement verndert das Objekt erst vor der Auswertung
//==============================================================
HG::Datum HG::Datum::operator-- (void)
{
  setJahr (nJahr-1);
  return *this;
}

//==============================================================
// Operator: Postdecrement
// Also erst eine Kopie erstellen, dann das Objekt selbst 
// abndern, die Kopie zurckgeben. Postinkrement verndert
// das Objekt erst nach der Auswertung
//==============================================================
HG::Datum HG::Datum::operator-- (int)
{
  HG::Datum oTemp = *this; 
  setJahr(nJahr-1);
  return oTemp;
}

//==============================================================
// Operator: Addition mit einem Integer
//==============================================================
HG::Datum HG::Datum::operator+ (int pToAdd)
{
  return Datum (nTag, nMonat, nJahr+pToAdd);
}

//==============================================================
// Operator: Subtraktion eines Integers
//==============================================================
HG::Datum HG::Datum::operator- (int pToSubtract)
{
  return Datum (nTag, nMonat, nJahr-pToSubtract);
}

//==============================================================
// Operator: Addition mit einem Integer - Version 1
//==============================================================
/*
int operator+ (int pToAdd, HG::Datum &pDatum)
{
  return pDatum.getJahr() + pToAdd;
}
*/

//==============================================================
// Operator: Addition mit einem Integer - Version 2
//==============================================================
HG::Datum operator+ (int pToAdd, HG::Datum &pDatum)
{
  return pDatum + pToAdd;
}

//==============================================================
// Operator: Subtraktion mit einem Integer - Version 1
//==============================================================
/*
int operator- (int pToSubtract, HG::Datum &pDatum)
{
  return pDatum.getJahr() - pToSubtract;
}
*/

//==============================================================
// Operator: Subtraktion mit einem Integer - Version 2
//==============================================================
HG::Datum operator- (int pToSubtract, HG::Datum &pDatum)
{
  return pDatum - pToSubtract;
}

//==============================================================
// Operator: Stream-Output
//==============================================================
ostream & operator<< (ostream & pStream, HG::Datum &pDatum)
{
  int nOldfill = pStream.fill ('0');
  pStream << setw(2) << pDatum.getTag()   << HG::Datum::getSeperator()
	      << setw(2) << pDatum.getMonat() << HG::Datum::getSeperator() 
          << pDatum.getJahr();
  pStream.fill (nOldfill);
  return pStream;
}

//==============================================================
// Operator: Zuweisung
//==============================================================
HG::Datum HG::Datum::operator= (HG::Datum &pToCopy)
{
  //------------------------------------------------------------
  // memcpy kopiert komplette Speicherbereiche
  // Bei Klassen mit Pointern kann nicht memcpy verwendet
  // werden, stattdessen muss das Ergebnis durch Einzel-
  // zuweisungen erbracht werden
  //------------------------------------------------------------
   memcpy (this, &pToCopy, sizeof(HG::Datum));
   return *this;
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Datum::operator== (HG::Datum &pToCompare)
{
  //------------------------------------------------------------
  // memcmp liefert 0 wenn die Inhalte gleich sind, sonst den
  // Subtraktionswert des ersten unterschiedlichen byte
  // Bei Klassen mit Pointern kann nicht memcmp verwendet
  // werden, stattdessen muss das Ergebnis durch Einzel-
  // vergleiche erbracht werden
  //------------------------------------------------------------
  return memcmp(this, &pToCompare, sizeof(HG::Datum)) == 0;
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Datum::operator!= (HG::Datum &pToCompare)
{
  //------------------------------------------------------------
  // memcmp liefert 0 wenn die Inhalte gleich sind, sonst den
  // Subtraktionswert des ersten unterschiedlichen byte
  // Bei Klassen mit Pointern kann nicht memcmp verwendet
  // werden, stattdessen muss das Ergebnis durch Einzel-
  // vergleiche erbracht werden
  //------------------------------------------------------------
  return !(*this == pToCompare);
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Datum::operator> (HG::Datum &pToCompare)
{
  //------------------------------------------------------------
  // kann ber Vergleiche == und > ermittelt werden, das spart
  // Programmierarbeit bei Klassen mit Pointern
  //------------------------------------------------------------
  return !((*this == pToCompare) || (*this < pToCompare));
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Datum::operator< (HG::Datum &pToCompare)
{
   if (nJahr < pToCompare.nJahr)
   {
      return true;
   }
   else
   {
      if (nJahr == pToCompare.nJahr)
      {
         if (nMonat < pToCompare.nMonat)
         {
            return true;
         }
         else
         {
            if (nMonat == pToCompare.nMonat)
            {
               if (nTag == pToCompare.nTag)
               {
                  return true;
               }
            }
         }
      }
   }
   return false;
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Datum::operator<= (HG::Datum &pToCompare)
{
  //------------------------------------------------------------
  // kann ber Vergleiche == und > ermittelt werden, das spart
  // Programmierarbeit bei Klassen mit Pointern
  //------------------------------------------------------------
  return !(*this > pToCompare);
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Datum::operator>= (HG::Datum &pToCompare)
{
  //------------------------------------------------------------
  // kann ber Vergleiche == und > ermittelt werden, das spart
  // Programmierarbeit bei Klassen mit Pointern
  //------------------------------------------------------------
  return !(*this < pToCompare);
}

//==============================================================
// maximale Anzahl von Tagen in einem Monat
//==============================================================
int HG::Datum::maxTageImMonat(int pMonat, int pJahr)
{
   switch (pMonat)
   {
     case 4:
     case 6:
     case 9:
     case 11: return 30;
     case 2:  if (pJahr % 4)      return 28;
              if (!(pJahr % 400)) return 29;
              if (!(pJahr % 100)) return 28;
   }
   return 31;
}

//==============================================================
// Addition von Jahren
//==============================================================
HG::Datum HG::Datum::addJahre(int pToAdd)
{
  setJahr(nJahr+pToAdd);
  return *this;
}

//==============================================================
// Subtraktion von Jahren
//==============================================================
HG::Datum HG::Datum::subJahre(int pToSub)
{
  setJahr(nJahr-pToSub);
  return *this;
}

//==============================================================
// Addition von Monaten
//==============================================================
HG::Datum HG::Datum::addMonate(int pToAdd)
{
  nMonat = nMonat + pToAdd;
  while (nMonat > 12)
  {
    nMonat = nMonat - 12;
    nJahr = nJahr + 1;
  }
  correct();
  return *this;
}

//==============================================================
// Subtraktion von Monaten
//==============================================================
HG::Datum HG::Datum::subMonate(int pToSub)
{
  nMonat = nMonat - pToSub;
  while (nMonat < 1)
  {
    nMonat = nMonat + 12;
    nJahr = nJahr - 1;
  }
  correct();
  return *this;
}

//==============================================================
// Addition von Tagen
//==============================================================
HG::Datum HG::Datum::addTage(int pToAdd)
{
  nTag = nTag + pToAdd;
  while (nTag > maxTageImMonat(nMonat, nJahr))
  {
    nTag = nTag - maxTageImMonat(nMonat, nJahr);
    nMonat = nMonat + 1;
    if (nMonat > 12)
    {
      nMonat = 1;
      nJahr = nJahr + 1;
    }
  }
  correct();
  return *this;
}

//==============================================================
// Subtraktion von Tagen
//==============================================================
HG::Datum HG::Datum::subTage(int pToSub)
{
  nTag = nTag - pToSub;
  while (nTag < 1)
  {
    nMonat = nMonat - 1;
    if (nMonat == 0)
    {
      nMonat = 12;
      nJahr = nJahr - 1;
    }
    nTag = nTag + maxTageImMonat(nMonat, nJahr);
  }
  correct();
  return *this;
}

//==============================================================
// Aktuelles Trennzeichen fr Datum zurckgeben
//==============================================================
char HG::Datum::getSeperator(void)
{
  return cSeperator;
}

//==============================================================
// Aktuelles Trennzeichen fr Datum setzen
// Besonders elegant: das alte Trennzeichen zurckgeben, so da
// man sich das beim Setzen direkt merken kann:
//--------------------------------------------------------------
// char altesZeichen = Datum::setSeperator('/');
//==============================================================
char HG::Datum::setSeperator(char pSeperator)
{
   char cOldSeperator = cSeperator;
   cSeperator = pSeperator;
   return cOldSeperator;
}

