#include "stdafx.h"
#include <time.h>
#include <iostream>
#include <iomanip>
#include "hg_uhrzeit.h"

using namespace std;

char HG::Uhrzeit::cSeperator = ':';

//==============================================================
// Standardconstructor, belegt das Uhrzeitobjekt mit der 
// aktuellen Uhrzeit (gem PC-Uhr)
//==============================================================
HG::Uhrzeit::Uhrzeit (void)
{
  setJetzt();
}

//==============================================================
// Constructor, mit Vorgaben
//==============================================================
HG::Uhrzeit::Uhrzeit (int pStunde, int pMinute)
{
  set (pStunde, pMinute);
}

//==============================================================
// Copy-Constructor (Referenz)
//==============================================================
HG::Uhrzeit::Uhrzeit (Uhrzeit &pUhrzeit)
{
  nStunde = pUhrzeit.nStunde;
  nMinute = pUhrzeit.nMinute;
}

//==============================================================
// Copy-Constructor (Pointer)
//==============================================================
HG::Uhrzeit::Uhrzeit (Uhrzeit *ptrUhrzeit)
{
  nStunde = ptrUhrzeit->nStunde;
  nMinute = ptrUhrzeit->nMinute;
}

//==============================================================
// Destructor
//==============================================================
HG::Uhrzeit::~Uhrzeit ()
{
}

//==============================================================
// Inhalt der Uhrzeit auf die aktuelle Zeit gem PC-Uhr setzen
//==============================================================
void HG::Uhrzeit::setJetzt (void)
{
  //------------------------------------------------------------
  // Timerticks aus PC-Uhr holen
  //------------------------------------------------------------
  time_t nTimerTicks;
  time(&nTimerTicks);                

  //------------------------------------------------------------
  // Konvertierung der Timerticks in das gem PC eingestellte
  // Zeitschema
  //------------------------------------------------------------
  struct tm *rcZeit = localtime(&nTimerTicks); 

  //------------------------------------------------------------
  // Aus der Struktur die relevanten Informationen herausnehmen
  // tm_mday = Tag im Monat, (1  31)
  // tm_mon  = Monat im Jahr (0  11; January = 0) (!!!!!) 
  // tm_year = Jahr          (Aktuelles Jahr minus 1900)
  //------------------------------------------------------------
  nStunde = rcZeit->tm_hour;             
  nMinute = rcZeit->tm_min;             
}

//==============================================================
// Getter des Members nStunde
//==============================================================
int HG::Uhrzeit::getStunde (void)
{
  return nStunde;
}

//==============================================================
// Setter des Members nStunde
//==============================================================
void HG::Uhrzeit::setStunde (int pStunde)
{
  nStunde = pStunde;
  correct();
}

//==============================================================
// Getter des Members nMinute
//==============================================================
int HG::Uhrzeit::getMinute (void)
{
  return nMinute;
}

//==============================================================
// Setter des Members nMinute
//==============================================================
void HG::Uhrzeit::setMinute (int pMinute)
{
  nMinute = pMinute;
  correct();
}

//==============================================================
// ganze Uhrzeit auf einmal setzen
//==============================================================
void HG::Uhrzeit::set (int pStunde, int pMinute)
{
  nStunde = pStunde;
  nMinute = pMinute;
  correct();
}

//==============================================================
// ganze Uhrzeit als long-Wert zurckgeben
//==============================================================
long HG::Uhrzeit::getLong (void)
{
  return nMinute + nStunde*100;
}

//==============================================================
// Uhrzeitregeln prfen
//==============================================================
void HG::Uhrzeit::correct (void)
{
  if (nStunde < 0) nStunde = 0;
  else if (nStunde > 23) nStunde = 23;

  if (nMinute < 0) nMinute = 0;
  else if (nMinute > 59) nMinute = 59;
}

//==============================================================
// Operator: Prinkrement
// Also erst das Objekt verndern und dann eine Kopie erstellen. 
// Prinkrement verndert das Objekt erst vor der Auswertung
//==============================================================
HG::Uhrzeit HG::Uhrzeit::operator++ (void)
{
  setStunde (nStunde+1);
  return *this;
}

//==============================================================
// Operator: Postincrement
// Also erst eine Kopie erstellen, dann das Objekt selbst 
// abndern, die Kopie zurckgeben. Postinkrement verndert
// das Objekt erst nach der Auswertung
//==============================================================
HG::Uhrzeit HG::Uhrzeit::operator++ (int)
{
  HG::Uhrzeit oTemp = *this; 
  setStunde (nStunde+1);
  return oTemp;
}

//==============================================================
// Operator: Prdekrement
// Also erst das Objekt verndern und dann eine Kopie erstellen. 
// Prinkrement verndert das Objekt erst vor der Auswertung
//==============================================================
HG::Uhrzeit HG::Uhrzeit::operator-- (void)
{
  setStunde (nStunde-1);
  return *this;
}

//==============================================================
// Operator: Postdecrement
// Also erst eine Kopie erstellen, dann das Objekt selbst 
// abndern, die Kopie zurckgeben. Postinkrement verndert
// das Objekt erst nach der Auswertung
//==============================================================
HG::Uhrzeit HG::Uhrzeit::operator-- (int)
{
  HG::Uhrzeit oTemp = *this; 
  setStunde (nStunde-1);
  return oTemp;
}

//==============================================================
// Operator: Stream-Output
//==============================================================
ostream & operator<< (ostream & pStream, HG::Uhrzeit &pUhrzeit)
{
  int nOldfill = pStream.fill ('0');
  pStream << setw(2) << pUhrzeit.getStunde() << HG::Uhrzeit::getSeperator() 
          << setw(2) << pUhrzeit.getMinute();
  pStream.fill (nOldfill);
  return pStream;
}

//==============================================================
// Operator: Zuweisung
//==============================================================
HG::Uhrzeit HG::Uhrzeit::operator= (HG::Uhrzeit &pToCopy)
{
  //------------------------------------------------------------
  // memcpy kopiert komplette Speicherbereiche
  // Bei Klassen mit Pointern kann nicht memcpy verwendet
  // werden, stattdessen muss das Ergebnis durch Einzel-
  // zuweisungen erbracht werden
  //------------------------------------------------------------
   memcpy (this, &pToCopy, sizeof(HG::Uhrzeit));
   return *this;
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Uhrzeit::operator== (HG::Uhrzeit &pToCompare)
{
  //------------------------------------------------------------
  // memcmp liefert 0 wenn die Inhalte gleich sind, sonst den
  // Subtraktionswert des ersten unterschiedlichen byte
  // Bei Klassen mit Pointern kann nicht memcmp verwendet
  // werden, stattdessen muss das Ergebnis durch Einzel-
  // vergleiche erbracht werden
  //------------------------------------------------------------
  return memcmp(this, &pToCompare, sizeof(HG::Uhrzeit)) == 0;
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Uhrzeit::operator!= (HG::Uhrzeit &pToCompare)
{
  //------------------------------------------------------------
  // memcmp liefert 0 wenn die Inhalte gleich sind, sonst den
  // Subtraktionswert des ersten unterschiedlichen byte
  // Bei Klassen mit Pointern kann nicht memcmp verwendet
  // werden, stattdessen muss das Ergebnis durch Einzel-
  // vergleiche erbracht werden
  //------------------------------------------------------------
  return !(*this == pToCompare);
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Uhrzeit::operator> (HG::Uhrzeit &pToCompare)
{
  //------------------------------------------------------------
  // kann ber Vergleiche == und > ermittelt werden, das spart
  // Programmierarbeit bei Klassen mit Pointern
  //------------------------------------------------------------
  return !((*this == pToCompare) || (*this < pToCompare));
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Uhrzeit::operator< (HG::Uhrzeit &pToCompare)
{
   if (nStunde < pToCompare.nStunde)
   {
      return true;
   }
   else
   {
      if (nStunde == pToCompare.nStunde)
      {
         if (nMinute < pToCompare.nMinute)
         {
            return true;
         }
      }
   }
   return false;
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Uhrzeit::operator<= (HG::Uhrzeit &pToCompare)
{
  //------------------------------------------------------------
  // kann ber Vergleiche == und > ermittelt werden, das spart
  // Programmierarbeit bei Klassen mit Pointern
  //------------------------------------------------------------
  return !(*this > pToCompare);
}

//==============================================================
// Operator: Vergleich
//==============================================================
bool HG::Uhrzeit::operator>= (HG::Uhrzeit &pToCompare)
{
  //------------------------------------------------------------
  // kann ber Vergleiche == und > ermittelt werden, das spart
  // Programmierarbeit bei Klassen mit Pointern
  //------------------------------------------------------------
  return !(*this < pToCompare);
}

//==============================================================
// Addition von Stunden
//==============================================================
int HG::Uhrzeit::addStunde(int pToAdd)
{
  if (pToAdd < 0) return subStunde(-pToAdd);

  int nUebertrag = 0;
  nStunde = nStunde + pToAdd;
  while (nStunde > 23)
  {
    nStunde = nStunde - 24;
    nUebertrag++;
  }
  return nUebertrag;
}

//==============================================================
// Subtraktion von Stunden
//==============================================================
int HG::Uhrzeit::subStunde(int pToSub)
{
  if (pToSub < 0) return addStunde(-pToSub);
 
  int nUebertrag = 0;
  nStunde = nStunde - pToSub;
  while (nStunde < 0)
  {
    nStunde = nStunde + 24;
    nUebertrag--;
  }
  return nUebertrag;
}

//==============================================================
// Addition von Minuten
//==============================================================
int HG::Uhrzeit::addMinute(int pToAdd)
{
  if (pToAdd < 0) return subMinute(-pToAdd);

  int nUebertrag = 0;
  nMinute = nMinute + pToAdd;
  while (nMinute > 59)
  {
    nMinute = nMinute - 60;
    nUebertrag++;
  }

  if (nUebertrag > 0)
  {
    nUebertrag = addStunde(nUebertrag);
  }
  return nUebertrag;
}

//==============================================================
// Subtraktion von Minuten
//==============================================================
int HG::Uhrzeit::subMinute(int pToSub)
{
  if (pToSub < 0) return addMinute(-pToSub);

  int nUebertrag = 0;
  nMinute = nMinute - pToSub;
  while (nMinute < 0)
  {
    nMinute = nMinute + 60;
    nUebertrag--;
  }

  if (nUebertrag < 0)
  {
    nUebertrag = subStunde(-nUebertrag);
  }
  return nUebertrag;
}

//==============================================================
// Operator: Subtraktion mit einem Integer
//==============================================================
HG::Uhrzeit HG::Uhrzeit::operator- (int pToSubtract)
{ 
  Uhrzeit oTemp (*this);
  oTemp.subMinute(pToSubtract);
  return oTemp;
}

//==============================================================
// Operator: Addition mit einem Integer
//==============================================================
HG::Uhrzeit HG::Uhrzeit::operator+ (int pToAdd)
{ 
  Uhrzeit oTemp (*this);
  oTemp.addMinute(pToAdd);
  return oTemp;
}

//==============================================================
// Operator: Subtraktion mit einem Integer
//==============================================================
HG::Uhrzeit operator- (int pToSubtract, HG::Uhrzeit &pUhrzeit)
{
  return pUhrzeit - pToSubtract;
}

//==============================================================
// Operator: Addition mit einem Integer
//==============================================================
HG::Uhrzeit operator+ (int pToAdd, HG::Uhrzeit &pUhrzeit)
{
  return pUhrzeit + pToAdd;
}

//==============================================================
// Aktuelles Trennzeichen fr Uhrzeit zurckgeben
//==============================================================
char HG::Uhrzeit::getSeperator(void)
{
  return cSeperator;
}

//==============================================================
// Aktuelles Trennzeichen fr Uhrzeit setzen
// Besonders elegant: das alte Trennzeichen zurckgeben, so da
// man sich das beim Setzen direkt merken kann:
//--------------------------------------------------------------
// char altesZeichen = Uhrzeit::setSeperator('/');
//==============================================================
char HG::Uhrzeit::setSeperator(char pSeperator)
{
   char cOldSeperator = cSeperator;
   cSeperator = pSeperator;
   return cOldSeperator;
}


